import { ClassValue } from 'clsx';
export { ClassValue, clsx } from 'clsx';

type ClassKey = "className";
type ClassProps = Partial<Record<ClassKey, ClassValue>>;
type OnnoClassValue = string | string[];
type OnnoClassProps = Record<ClassKey, OnnoClassValue>;
type OnnoClassMap = Record<string, OnnoClassValue>;
type OnnoVariants = Record<string, OnnoClassValue | OnnoClassMap>;
type OnnoDefaults<T extends OnnoVariants> = {
    [K in keyof T]?: T[K] extends OnnoClassMap ? keyof T[K] : boolean;
};
type OnnoCompound<T extends OnnoVariants> = {
    [K in keyof T]?: T[K] extends OnnoClassMap ? keyof T[K] | Array<keyof T[K]> : boolean;
} & OnnoClassProps;
interface OnnoConfig<T extends OnnoVariants> {
    baseline?: OnnoClassValue;
    compound?: OnnoCompound<T>[];
    defaults?: OnnoDefaults<T>;
    variants: T;
}
type OnnoOptions<T extends OnnoVariants> = OnnoDefaults<T> & ClassProps;
type OnnoFunction<T extends OnnoVariants> = (options?: OnnoOptions<T>) => string;
type OnnoFactory = <T extends OnnoVariants>(config: OnnoConfig<T>) => OnnoFunction<T>;
type OnnoVariantProps<F extends OnnoFunction<any>> = Omit<Exclude<Parameters<F>[0], undefined>, ClassKey>;
type OnnoProps<F extends OnnoFunction<any>, K extends keyof OnnoVariantProps<F> = never> = OnnoVariantProps<F> & Required<Pick<OnnoVariantProps<F>, K>> & ClassProps;

declare const onno: OnnoFactory;

export { ClassKey, ClassProps, OnnoClassMap, OnnoClassProps, OnnoClassValue, OnnoCompound, OnnoConfig, OnnoDefaults, OnnoFactory, OnnoFunction, OnnoOptions, OnnoProps, OnnoVariants, onno };
